"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createGameOfLife = void 0;
var src_1 = require("../../src");
var lambda_1 = require("../../src/lambda/lambda");
var ALIVE_VALUE = '*';
var createAliveCell = function () { return (0, src_1.right)(ALIVE_VALUE); };
var DEAD_VALUE = '.';
var createDeadCell = function () { return (0, src_1.left)(DEAD_VALUE); };
var createCheckPattern = function (row, col) { return [
    [row - 1, col - 1],
    [row - 1, col],
    [row - 1, col + 1],
    [row, col + 1],
    [row, col - 1],
    [row + 1, col - 1],
    [row + 1, col],
    [row + 1, col + 1],
]; };
var createInitialState = function (width, height) { return ({
    grid: (0, lambda_1.repeat)(height, function () { return (0, lambda_1.repeat)(width, createDeadCell); }),
}); };
var createGameOfLife = function (width, height) {
    var state = createInitialState(width, height);
    var getCell = function (row, col) { return (0, src_1.maybe)(state.grid[row]).flatMap(function (it) { return (0, src_1.maybe)(it[col]); }); };
    var setLivingCell = function (row, col) {
        (0, src_1.maybe)(state.grid[row]).fold(function () { }, function (it) {
            it[col] = createAliveCell();
        });
    };
    var countLivingNeighbours = function (row, col) {
        return createCheckPattern(row, col).reduce(function (acc, _a) {
            var rowIndex = _a[0], colIndex = _a[1];
            return getCell(rowIndex, colIndex).fold(function () { return acc; }, function (it) {
                return it.fold(function () { return acc; }, function () { return acc + 1; });
            });
        }, 0);
    };
    var isDead = function (row, col) {
        return getCell(row, col).fold(function () { return false; }, function (it) { return it.isLeft; });
    };
    var isAlive = function (row, col) {
        return getCell(row, col).fold(function () { return false; }, function (it) { return it.isRight; });
    };
    var createNextGeneration = function () {
        return state.grid.map(function (row, rowIndex) {
            return row.map(function (cell, colIndex) {
                var livingNeighbours = countLivingNeighbours(rowIndex, colIndex);
                if (livingNeighbours === 3)
                    return createAliveCell();
                return cell.flatMap(function (it) { return (livingNeighbours < 2 || livingNeighbours > 3 ? createDeadCell() : (0, src_1.right)(it)); });
            });
        });
    };
    var computeNextGeneration = function () {
        state.grid = createNextGeneration();
    };
    var dumpGrid = function () {
        return state.grid.map(function (row) { return row.map(function (cell) { return cell.getOrElse(function () { return DEAD_VALUE; }); }).join(''); }).join('\n');
    };
    return {
        countLivingNeighbours: countLivingNeighbours,
        setLivingCell: setLivingCell,
        isDead: isDead,
        isAlive: isAlive,
        computeNextGeneration: computeNextGeneration,
        dumpGrid: dumpGrid,
    };
};
exports.createGameOfLife = createGameOfLife;
//# sourceMappingURL=game-of-life.js.map