"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var game_of_life_1 = require("./game-of-life");
describe('Game of life', function () {
    var gameOfLife;
    beforeEach(function () {
        gameOfLife = (0, game_of_life_1.createGameOfLife)(8, 4);
    });
    it('foundsNoLivingNeighboursInAnEmptyGrid', function () {
        var neighboursCount = gameOfLife.countLivingNeighbours(1, 4);
        expect(neighboursCount).toBe(0);
    });
    it('foundsOneLivingNeighbour', function () {
        gameOfLife.setLivingCell(0, 3);
        var neighboursCount = gameOfLife.countLivingNeighbours(1, 4);
        expect(neighboursCount).toBe(1);
    });
    it('foundsTwoLivingNeighbours', function () {
        gameOfLife.setLivingCell(0, 3);
        gameOfLife.setLivingCell(0, 4);
        var neighboursCount = gameOfLife.countLivingNeighbours(1, 4);
        expect(neighboursCount).toBe(2);
    });
    it('foundsThreeLivingNeighbours', function () {
        gameOfLife.setLivingCell(0, 3);
        gameOfLife.setLivingCell(0, 4);
        gameOfLife.setLivingCell(0, 5);
        var neighboursCount = gameOfLife.countLivingNeighbours(1, 4);
        expect(neighboursCount).toBe(3);
    });
    it('foundsFourLivingNeighbours', function () {
        gameOfLife.setLivingCell(0, 3);
        gameOfLife.setLivingCell(0, 4);
        gameOfLife.setLivingCell(0, 5);
        gameOfLife.setLivingCell(1, 5);
        var neighboursCount = gameOfLife.countLivingNeighbours(1, 4);
        expect(neighboursCount).toBe(4);
    });
    it('foundsFiveLivingNeighbours', function () {
        gameOfLife.setLivingCell(0, 3);
        gameOfLife.setLivingCell(0, 4);
        gameOfLife.setLivingCell(0, 5);
        gameOfLife.setLivingCell(1, 5);
        gameOfLife.setLivingCell(2, 5);
        var neighboursCount = gameOfLife.countLivingNeighbours(1, 4);
        expect(neighboursCount).toBe(5);
    });
    it('foundsSixLivingNeighbours', function () {
        gameOfLife.setLivingCell(0, 3);
        gameOfLife.setLivingCell(0, 4);
        gameOfLife.setLivingCell(0, 5);
        gameOfLife.setLivingCell(1, 5);
        gameOfLife.setLivingCell(2, 5);
        gameOfLife.setLivingCell(2, 4);
        var neighboursCount = gameOfLife.countLivingNeighbours(1, 4);
        expect(neighboursCount).toBe(6);
    });
    it('foundsSevenLivingNeighbours', function () {
        gameOfLife.setLivingCell(0, 3);
        gameOfLife.setLivingCell(0, 4);
        gameOfLife.setLivingCell(0, 5);
        gameOfLife.setLivingCell(1, 5);
        gameOfLife.setLivingCell(2, 5);
        gameOfLife.setLivingCell(2, 4);
        gameOfLife.setLivingCell(2, 3);
        var neighboursCount = gameOfLife.countLivingNeighbours(1, 4);
        expect(neighboursCount).toBe(7);
    });
    it('foundsHeightLivingNeighbours', function () {
        gameOfLife.setLivingCell(0, 3);
        gameOfLife.setLivingCell(0, 4);
        gameOfLife.setLivingCell(0, 5);
        gameOfLife.setLivingCell(1, 5);
        gameOfLife.setLivingCell(2, 5);
        gameOfLife.setLivingCell(2, 4);
        gameOfLife.setLivingCell(2, 3);
        gameOfLife.setLivingCell(1, 3);
        var neighboursCount = gameOfLife.countLivingNeighbours(1, 4);
        expect(neighboursCount).toBe(8);
    });
    it('foundsThreeLivingNeighboursForTheTopLeftCornerCell', function () {
        gameOfLife.setLivingCell(0, 1);
        gameOfLife.setLivingCell(1, 1);
        gameOfLife.setLivingCell(1, 0);
        var neighboursCount = gameOfLife.countLivingNeighbours(0, 0);
        expect(neighboursCount).toBe(3);
    });
    it('foundsThreeLivingNeighboursForTheBottomRightCornerCell', function () {
        gameOfLife.setLivingCell(3, 6);
        gameOfLife.setLivingCell(2, 6);
        gameOfLife.setLivingCell(2, 7);
        var neighboursCount = gameOfLife.countLivingNeighbours(3, 7);
        expect(neighboursCount).toBe(3);
    });
    it('aCellWithFewerThanTwoNeighboursDies', function () {
        gameOfLife.setLivingCell(0, 0);
        gameOfLife.setLivingCell(0, 1);
        gameOfLife.computeNextGeneration();
        expect(gameOfLife.isDead(0, 0)).toBeTruthy();
    });
    it('aCellWithAtLeastTwoNeighboursLives', function () {
        gameOfLife.setLivingCell(0, 0);
        gameOfLife.setLivingCell(0, 1);
        gameOfLife.setLivingCell(1, 1);
        gameOfLife.computeNextGeneration();
        expect(gameOfLife.isAlive(0, 0)).toBeTruthy();
    });
    it('aCellWithMoreThanThreeNeighboursDies', function () {
        gameOfLife.setLivingCell(1, 4);
        gameOfLife.setLivingCell(0, 3);
        gameOfLife.setLivingCell(0, 4);
        gameOfLife.setLivingCell(0, 5);
        gameOfLife.setLivingCell(1, 5);
        gameOfLife.computeNextGeneration();
        expect(gameOfLife.isDead(1, 4)).toBeTruthy();
    });
    it('aDeadCellWithThreeNeighboursGetsAlive', function () {
        gameOfLife.setLivingCell(0, 3);
        gameOfLife.setLivingCell(0, 4);
        gameOfLife.setLivingCell(0, 5);
        gameOfLife.computeNextGeneration();
        expect(gameOfLife.isAlive(1, 4)).toBeTruthy();
    });
    it('testAnHandMadeGrid', function () {
        gameOfLife.setLivingCell(1, 4);
        gameOfLife.setLivingCell(2, 3);
        gameOfLife.setLivingCell(2, 4);
        gameOfLife.computeNextGeneration();
        expect(gameOfLife.dumpGrid()).toBe("\n........\n...**...\n...**...\n........\n\t\t".trim());
    });
});
//# sourceMappingURL=game-of-life.test.js.map