"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createGame = void 0;
var maybe_1 = require("../../src/maybe/maybe");
var SCORES = ['love', 'fifteen', 'thirty', 'forty'];
var MAX_SCORE_INDEX = SCORES.length - 1;
var capitalize = function (s) { return s.charAt(0).toUpperCase() + s.slice(1); };
var getAllScoreText = function (player) {
    return (0, maybe_1.maybe)(SCORES[player.scoreIndex])
        .flatMap(capitalize)
        .getOrElse(function () { return ''; });
};
var createPlayer = function (name) { return ({
    name: name,
    scoreIndex: 0,
}); };
var fromBoolean = function (expression) { return (0, maybe_1.maybe)(expression || undefined); };
var createGame = function (firstPlayerName, secondPlayerName) {
    var state = {
        players: [createPlayer(firstPlayerName), createPlayer(secondPlayerName)],
    };
    var _a = state.players, playerOne = _a[0], playerTwo = _a[1];
    var getRank = function () { return playerOne.scoreIndex - playerTwo.scoreIndex; };
    var canWin = function (player) { return player.scoreIndex > MAX_SCORE_INDEX; };
    var allCanWin = function () { return fromBoolean(state.players.every(canWin)); };
    var isDraw = function () { return fromBoolean(getRank() === 0); };
    var isAll = function () { return fromBoolean(state.players.every(function (it) { return it.scoreIndex < MAX_SCORE_INDEX; })); };
    var isDeuce = function () { return fromBoolean(state.players.every(function (it) { return it.scoreIndex >= MAX_SCORE_INDEX; })); };
    var score = function (player) {
        player.scoreIndex++;
    };
    var handleDraw = function () {
        return isDeuce()
            .flatMap(function () { return 'Deuce'; })
            .mapLeft(function () { return isAll().flatMap(function () { return "".concat(getAllScoreText(playerOne), " all"); }); });
    };
    var handleAdvantage = function () {
        var rank = getRank();
        var spread = Math.abs(rank);
        var topPlayer = rank > 0 ? playerOne : playerTwo;
        return allCanWin()
            .flatMap(function () { return (spread === 1 ? "Advantage ".concat(topPlayer.name) : (0, maybe_1.nothing)()); })
            .mapLeft(function () { return (canWin(topPlayer) && spread >= 2 ? (0, maybe_1.just)("".concat(topPlayer.name, " wins")) : (0, maybe_1.nothing)()); });
    };
    return {
        getScore: function () {
            return isDraw()
                .flatMap(handleDraw)
                .mapLeft(handleAdvantage)
                .getOrElse(function () { return "".concat(getAllScoreText(playerOne), ",").concat(getAllScoreText(playerTwo)); });
        },
        playerOneScores: function () { return score(playerOne); },
        playerTwoScores: function () { return score(playerTwo); },
    };
};
exports.createGame = createGame;
//# sourceMappingURL=tennis-game.js.map